/* Field correction in Code Saturne */
/* Based on Code Saturne examples */
/* Original name: cs_user_extra_operations.c */

/* Headers */
#include "cs_defs.h"

/* Standard C headers */
#include <assert.h>
#include <math.h>

/* MPI headers */
#if defined(HAVE_MPI)
#include <mpi.h>
#endif

/* PLE library headers */
#include <ple_coupling.h>

/* Local headers */
/* Code Saturne BFT */
#include "bft_mem.h"
#include "bft_error.h"
#include "bft_printf.h"
/* Code Saturne Common */
#include "cs_base.h"
#include "cs_field.h"
#include "cs_field_pointer.h"
#include "cs_field_operator.h"
#include "cs_geom.h"
#include "cs_mesh.h"
#include "cs_mesh_quantities.h"
#include "cs_halo.h"
#include "cs_halo_perio.h"
#include "cs_log.h"
#include "cs_parall.h"
#include "cs_parameters.h"
#include "cs_physical_constants.h"
#include "cs_prototypes.h"
#include "cs_rotation.h"
#include "cs_time_moment.h"
#include "cs_time_step.h"
#include "cs_turbomachinery.h"
#include "cs_selector.h"
#include "cs_post.h"
/* Code Saturne headers for current file */
#include "cs_prototypes.h"

BEGIN_C_DECLS

/* Field correction */
void cs_user_extra_operations(cs_domain_t *domain)
{
 bool isPrsLim,isVelLim;
 int iCoord,nCoord;
 double prsMin,prsMinRq,prsMax,prsMaxRq,velMin,velMinRq,velMax,velMaxRq;
 cs_lnum_t iCell,nCell,nCellExt,nPrsLim,nVelLim;
 cs_real_3_t* cellVel;
 nCoord=3; /* Number of dimensions */
 nCell=cs_glob_mesh->n_cells; /* Number of regular cells */
 nCellExt=cs_glob_mesh->n_cells_with_ghosts; /* Number of cells including ghost cells */
 /* Clipping options */
 prsMinRq=-1000; /* Minimum allowable pressure [Pa] */
 prsMaxRq=1000; /* Maximum allowable pressure [Pa] */
 velMinRq=-50; /* Minimum allowable velocity component [m/s] */
 velMaxRq=50; /* Maximum allowable velocity component [m/s] */
 /* Processing pressure field */
 cs_field_t *prsField=cs_field_by_name_try("pressure"); /* Pointer to pressure field */
 if (prsField!=NULL) /* Pressure field present */
 {
  nPrsLim=0; /* Number of cells with limited pressure */
  prsMin=1E100; /* Minimum pressure [Pa] */
  prsMax=-1E100; /* Maximum pressure [Pa] */
  for (iCell=0;iCell<nCell;iCell++) /* Cell loop */
  {
   isPrsLim=false; /* Pressure limiting indicator */
   /* Check pressure in current cell and clip if needed */
   if (prsField->val[iCell]<prsMin) prsMin=prsField->val[iCell]; /* Initial minimum pressure [Pa] */
   if (prsField->val[iCell]<prsMinRq) /* Pressure is lower than minimum */
   {
    isPrsLim=true; /* Pressure limiting indicator */
    prsField->val[iCell]=prsMinRq; /* Clipping with minimum */
   };
   if (prsField->val[iCell]>prsMax) prsMax=prsField->val[iCell]; /* Initial maximum pressure [Pa] */
   if (prsField->val[iCell]>prsMaxRq) /* Pressure is higher than maximum */
   {
    isPrsLim=true; /* Pressure limiting indicator */
    prsField->val[iCell]=prsMaxRq; /* Clipping with maximum */
   };
   if (isPrsLim) /* Pressure was limiting in current cell */
   {
    nPrsLim=nPrsLim+1; /* Number of cells with limited pressure */
   };
  };
  if (nPrsLim>0) printf("\n[!] Warning! Pressure was limited in [%d] cells, range was [%.0f...%.0f] Pa\n",(int)nPrsLim,prsMin,prsMax);
 };
 /* Processing velocity field */
 cs_field_t *velField=cs_field_by_name_try("velocity"); /* Pointer to velocity field */
 cellVel=(cs_real_3_t*)velField->val; /* Pointer to velocitie in cells */
 if (velField!=NULL) /* Velocity field present */
 {
  nVelLim=0; /* Number of cels with limited velocity */
  velMin=1E100; /* Minimum velocity [m/s] */
  velMax=-1E100; /* Maximum velocity [m/s] */
  for (iCell=0;iCell<nCell;iCell++) /* Cell loop */
  {
   isVelLim=false; /* Velocity limiting indicator */
   for (iCoord=0;iCoord<nCoord;iCoord++) /* Coordinate loop */
   {
    /* Check velocity in current cell and clip if needed */
    if (cellVel[iCell][iCoord]<velMin) velMin=cellVel[iCell][iCoord]; /* Initial minimum velocity [m/s] */
    if (cellVel[iCell][iCoord]<velMinRq) /* Velocity component is lower than minimum */
    {
     isVelLim=true; /* Velocity limiting indicator */
     cellVel[iCell][iCoord]=velMinRq; /* Clipping with minimum */
    };
    if (cellVel[iCell][iCoord]>velMax) velMax=cellVel[iCell][iCoord]; /* Initial maximum velocity [m/s] */
    if (cellVel[iCell][iCoord]>velMaxRq) /* Velocity component is higher than maximum */
    {
     isVelLim=true; /* Velocity limiting indicator */
     cellVel[iCell][iCoord]=velMaxRq; /* Clipping with maximum */
    };
   };
   if (isVelLim) /* Velocity was limited in current cell */
   {
    nVelLim=nVelLim+1; /* Number of cels with limited velocity */
   };
  }
  if (nVelLim>0) printf("\n[!] Warning! Velocity was limited in [%d] cells, range was [%.0f...%.0f] m/s\n",(int)nVelLim,velMin,velMax);
 };
}

END_C_DECLS
